/********************************************************************\
 * gnc-helpers.c -- gnucash g-wrap helper functions                 *
 * Copyright (C) 2000 Linas Vepstas                                 *
 *                                                                  *
 * This program is free software; you can redistribute it and/or    *
 * modify it under the terms of the GNU General Public License as   *
 * published by the Free Software Foundation; either version 2 of   *
 * the License, or (at your option) any later version.              *
 *                                                                  *
 * This program is distributed in the hope that it will be useful,  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    *
 * GNU General Public License for more details.                     *
 *                                                                  *
 * You should have received a copy of the GNU General Public License*
 * along with this program; if not, contact:                        *
 *                                                                  *
 * Free Software Foundation           Voice:  +1-617-542-5942       *
 * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
 * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
 *                                                                  *
\********************************************************************/

#include "config.h"

#include "gnc-helpers.h"
#include "guile-util.h"
#include "gnc.h"
#include "messages.h"

#include <libguile.h>
#include <string.h>

#include <g-wrap-runtime-guile.h>

/* These will eventually go into (g-wrapped glib). */

static SCM
glist_to_scm_list_helper(GList *glist, SCM wct)
{
  SCM list = SCM_EOL;
  GList *node;

  for (node = glist; node; node = node->next)
    list = gh_cons (gw_wcp_assimilate_ptr(node->data, wct), list);

  return gh_reverse (list);
}

SCM
gnc_glist_to_scm_list(GList *glist, SCM wct)
{
  SCM_ASSERT(gw_wct_p(wct), wct, SCM_ARG1, "gnc_glist_to_scm_list");
  return(glist_to_scm_list_helper(glist, wct));
}

GList*
gnc_scm_list_to_glist(SCM rest)
{
  GList *result = NULL;
  SCM scm_item;
  
  SCM_ASSERT(gh_list_p(rest), rest, SCM_ARG1, "gnc_scm_list_to_glist");

  while(!gh_null_p(rest)) {
    void *item;
    scm_item = gh_car(rest);
    if(!gw_wcp_p(scm_item)) {
      scm_misc_error("gnc_scm_list_to_glist",
                     "Item in list not a gw:wcp.", scm_item);
    }
    item = gw_wcp_get_ptr(scm_item);
    result = g_list_prepend(result, item);
    rest = gh_cdr(rest);
  }
  return g_list_reverse(result);
}

static SCM
glist_map_helper(GList *glist, SCM wct, SCM thunk)
{
  SCM list = SCM_EOL;
  GList *node;

  for (node = glist; node; node = node->next)
    list = gh_cons (gh_call1(thunk, gw_wcp_assimilate_ptr(node->data, wct)),
                    list);

  return gh_reverse (list);
}

SCM
gnc_glist_scm_map(SCM wct, SCM thunk, GList* glist)
{
  SCM_ASSERT(gw_wct_p(wct), wct, SCM_ARG1, "gnc_glist_map");
  SCM_ASSERT(gh_procedure_p(thunk), thunk, SCM_ARG2, "gnc_glist_scm_map");  
  return(glist_map_helper(glist, wct, thunk));
}

void
gnc_glist_scm_for_each(SCM wct, SCM thunk, GList *glist)
{
  GList *lp;
  SCM_ASSERT(gw_wct_p(wct), wct, SCM_ARG1, "gnc_glist_map");
  SCM_ASSERT(gh_procedure_p(thunk), thunk, SCM_ARG2, "gnc_glist_scm_for_each");  
  for(lp = glist; lp; lp = lp->next) {
    gh_call1(thunk, gw_wcp_assimilate_ptr(lp->data, wct));
  }
}


int
gnc_guid_type(GUID guid)
{
  return xaccGUIDType(&guid);
}

GUID
gnc_account_get_guid(Account *account) {
  return *xaccAccountGetGUID(account);
}

Account *
gnc_account_lookup(GUID guid) {
  return xaccAccountLookup(&guid);
}

GUID
gnc_trans_get_guid(Transaction *trans) {
  return *xaccTransGetGUID(trans);
}

Transaction *
gnc_trans_lookup(GUID guid) {
  return xaccTransLookup(&guid);
}

GUID
gnc_split_get_guid(Split *split) {
  return *xaccSplitGetGUID(split);
}

Split *
gnc_split_lookup(GUID guid) {
  return xaccSplitLookup(&guid);
}

int
gnc_trans_is_common_currency(Transaction *trans, gnc_commodity * currency)
{
  return xaccTransIsCommonCurrency(trans, currency) != NULL;
}

Timespec
gnc_transaction_get_date_posted(Transaction *t) {
  Timespec result;
  xaccTransGetDateTS(t, &result);
  return(result);
}

Timespec
gnc_transaction_get_date_entered(Transaction *t) {
  Timespec result;
  xaccTransGetDateEnteredTS(t, &result);
  return(result);
}

Timespec
gnc_split_get_date_reconciled(Split *s) {
  Timespec result;
  xaccSplitGetDateReconciledTS(s, &result);
  return(result);
}

void
gnc_transaction_set_date_posted(Transaction *t, const Timespec d) {
  xaccTransSetDateTS(t, &d);
}

void
gnc_transaction_set_date_entered(Transaction *t, const Timespec d) {
  xaccTransSetDateEnteredTS(t, &d);
}

void
gnc_transaction_set_date(Transaction *t, Timespec ts)
{
  xaccTransSetDateTS(t, &ts);
}

char *
gnc_gettext_helper(const char *string)
{
  return strdup(_(string));
}

SCM
gnc_timespec2timepair(Timespec t)
{
  SCM secs;
  SCM nsecs;

  secs = gnc_gint64_to_scm(t.tv_sec);
  nsecs = gh_long2scm(t.tv_nsec);
  return(gh_cons(secs, nsecs));
}

Timespec
gnc_timepair2timespec(SCM x)
{
  Timespec result = {0,0};
  if (gnc_timepair_p (x))
  {
    result.tv_sec = gnc_scm_to_gint64(gh_car(x));
    result.tv_nsec = gh_scm2long(gh_cdr(x));
  }
  return(result);
}

int
gnc_timepair_p(SCM x)
{
  return(gh_pair_p(x) &&
         gnc_gh_gint64_p(gh_car(x)) &&
         gnc_gh_gint64_p(gh_cdr(x)));
}

SCM
gnc_guid2scm(GUID guid) {
  char *string;
  SCM result;

  string = guid_to_string(&guid);
  if (string == NULL)
    return SCM_UNDEFINED;

  result = gh_str02scm(string);

  g_free(string);

  return result;
}

GUID
gnc_scm2guid(SCM guid_scm) {
  char string[GUID_ENCODING_LENGTH + 1];
  GUID guid;

  gh_get_substr(guid_scm, string, 0, GUID_ENCODING_LENGTH);
  string[GUID_ENCODING_LENGTH] = '\0';

  string_to_guid(string, &guid);

  return guid;
}

int
gnc_guid_p(SCM guid_scm) {
  char string[GUID_ENCODING_LENGTH + 1];
  GUID guid;

  if (!gh_string_p(guid_scm))
    return (0 == 1);

  gh_get_substr(guid_scm, string, 0, GUID_ENCODING_LENGTH);
  string[GUID_ENCODING_LENGTH] = '\0';

  return string_to_guid(string, &guid);
}



/********************************************************************
 * type converters for query API  
 ********************************************************************/

SCM
gnc_queryop2scm(QueryOp val) {
  SCM car = gh_symbol2scm("query-op");
  SCM cdr = gh_int2scm(val);

  return gh_cons(car, cdr);
}

QueryOp
gnc_scm2queryop(SCM val) {
  QueryOp retval=gh_scm2int(gh_cdr(val));
  return retval;
}

int
gnc_queryop_p(SCM op) {
  char * symbol = gh_symbol2newstr(gh_car(op), NULL);
  int retval;
  if(!strcmp(symbol, "query-op")) {
    retval = 1;
  }
  else {
    retval = 0;
  }
  g_free(symbol);
  return retval;
}

SCM
gnc_sorttype2scm(sort_type_t val) {
  SCM car = gh_symbol2scm("sort-type");
  SCM cdr = gh_int2scm(val);

  return gh_cons(car, cdr);
}

sort_type_t
gnc_scm2sorttype(SCM val) {
  sort_type_t retval=gh_scm2int(gh_cdr(val));
  return retval;
}

int
gnc_sorttype_p(SCM op) {
  char * symbol = gh_symbol2newstr(gh_car(op), NULL);
  int retval;
  if(!strcmp(symbol, "sort-type")) {
    retval = 1;
  }
  else {
    retval = 0;
  }
  g_free(symbol);
  return retval;
}


SCM
gnc_termtype2scm(pd_type_t val) {
  SCM car = gh_symbol2scm("query-term-type");
  SCM cdr = gh_int2scm(val);

  return gh_cons(car, cdr);
}

pd_type_t
gnc_scm2termtype(SCM val) {
  pd_type_t retval=gh_scm2int(gh_cdr(val));
  return retval;
}

int
gnc_termtype_p(SCM op) {
  char * symbol = gh_symbol2newstr(gh_car(op), NULL);
  int retval;
  if(!strcmp(symbol, "query-term-type")) {
    retval = 1;
  }
  else {
    retval = 0;
  }
  g_free(symbol);
  return retval;
}


SCM
gnc_amtmatchsgn2scm(amt_match_sgn_t val) {
  SCM car = gh_symbol2scm("amt-match-sign");
  SCM cdr = gh_int2scm(val);

  return gh_cons(car, cdr);
}

amt_match_sgn_t
gnc_scm2amtmatchsgn(SCM val) {
  pd_type_t retval=gh_scm2int(gh_cdr(val));
  return retval;
}

int
gnc_amtmatchsgn_p(SCM op) {
  char * symbol = gh_symbol2newstr(gh_car(op), NULL);
  int retval;
  if(!strcmp(symbol, "amt-match-sign")) {
    retval = 1;
  }
  else {
    retval = 0;
  }
  g_free(symbol);
  return retval;
}


SCM
gnc_amtmatch2scm(amt_match_t val) {
  SCM car = gh_symbol2scm("amt-match-how");
  SCM cdr = gh_int2scm(val);

  return gh_cons(car, cdr);
}

amt_match_t
gnc_scm2amtmatch(SCM val) {
  pd_type_t retval=gh_scm2int(gh_cdr(val));
  return retval;
}

int
gnc_amtmatch_p(SCM op) {
  char * symbol = gh_symbol2newstr(gh_car(op), NULL);
  int retval;
  if(!strcmp(symbol, "amt-match-how")) {
    retval = 1;
  }
  else {
    retval = 0;
  }
  g_free(symbol);
  return retval;
}


SCM
gnc_acctmatch2scm(acct_match_t val) {
  SCM car = gh_symbol2scm("acct-match-how");
  SCM cdr = gh_int2scm(val);

  return gh_cons(car, cdr);
}

acct_match_t
gnc_scm2acctmatch(SCM val) {
  pd_type_t retval=gh_scm2int(gh_cdr(val));
  return retval;
}

int
gnc_acctmatch_p(SCM op) {
  char * symbol = gh_symbol2newstr(gh_car(op), NULL);
  int retval;
  if(!strcmp(symbol, "acct-match-how")) {
    retval = 1;
  }
  else {
    retval = 0;
  }
  g_free(symbol);
  return retval;
}

/* Type converters for GNCPrintAmountInfo */
SCM
gnc_printinfo2scm(GNCPrintAmountInfo info)
{
  SCM info_scm = SCM_EOL;

  info_scm = gh_cons (gh_bool2scm (info.monetary), info_scm);
  info_scm = gh_cons (gh_bool2scm (info.use_locale), info_scm);
  info_scm = gh_cons (gh_bool2scm (info.use_symbol), info_scm);
  info_scm = gh_cons (gh_bool2scm (info.use_separators), info_scm);

  info_scm = gh_cons (gh_int2scm (info.min_decimal_places), info_scm);
  info_scm = gh_cons (gh_int2scm (info.max_decimal_places), info_scm);

  info_scm = gh_cons (gnc_commodity_to_scm (info.commodity), info_scm);

  info_scm = gh_cons (gh_symbol2scm ("print-info"), info_scm);

  return info_scm;
}

GNCPrintAmountInfo
gnc_scm2printinfo(SCM info_scm)
{
  GNCPrintAmountInfo info;

  /* skip type */
  info_scm = gh_cdr (info_scm);
  info.commodity = gnc_scm_to_commodity (gh_car (info_scm));

  info_scm = gh_cdr (info_scm);
  info.max_decimal_places = gh_scm2int (gh_car (info_scm));

  info_scm = gh_cdr (info_scm);
  info.min_decimal_places = gh_scm2int (gh_car (info_scm));

  info_scm = gh_cdr (info_scm);
  info.use_separators = gh_scm2bool (gh_car (info_scm));

  info_scm = gh_cdr (info_scm);
  info.use_symbol = gh_scm2bool (gh_car (info_scm));

  info_scm = gh_cdr (info_scm);
  info.use_locale = gh_scm2bool (gh_car (info_scm));

  info_scm = gh_cdr (info_scm);
  info.monetary = gh_scm2bool (gh_car (info_scm));

  return info;
}

int
gnc_printinfo_p(SCM info_scm)
{
  char *symbol;
  int retval;

  if (!gh_list_p(info_scm) || gh_null_p(info_scm))
    return 0;

  info_scm = gh_car (info_scm);
  if (!gh_symbol_p (info_scm))
    return 0;

  symbol = gh_symbol2newstr (info_scm, NULL);
  if (symbol == NULL)
    return 0;

  retval = strcmp (symbol, "print-info") == 0;

  free (symbol);

  return retval;
}

static int
gnc_scm_traversal_adapter(Transaction *t, void *data)
{
  static SCM trans_type = SCM_BOOL_F;
  SCM result;
  SCM scm_trans;
  SCM thunk = *((SCM *) data);

  if(trans_type == SCM_BOOL_F) {
    trans_type = gh_eval_str("<gnc:Transaction*>");
    /* don't really need this - types are bound globally anyway. */
    if(trans_type != SCM_BOOL_F) scm_protect_object(trans_type);
  }
  
  scm_trans = gw_wcp_assimilate_ptr(t, trans_type);
  result = gh_call1(thunk, scm_trans);

  return (result != SCM_BOOL_F);
}

gboolean
gnc_scmGroupStagedTransactionTraversal(AccountGroup *grp,
                                       unsigned int new_marker,
                                       SCM thunk)
{
  return xaccGroupStagedTransactionTraversal(grp, new_marker,
                                             gnc_scm_traversal_adapter,
                                             &thunk);
}

gboolean
gnc_scmAccountStagedTransactionTraversal(Account *a,
                                         unsigned int new_marker,
                                         SCM thunk) 
{
  return xaccAccountStagedTransactionTraversal(a, new_marker,
					       gnc_scm_traversal_adapter,
                                               &thunk);
}

SCM
gnc_gint64_to_scm(const gint64 x)
{
#if GUILE_LONG_LONG_OK 
  return scm_long_long2num(x);
#else
  const gchar negative_p = (x < 0);
  const guint64 magnitude = negative_p ? -x : x;
  const guint32 lower_half = (guint32) (magnitude & 0xFFFFFFFF);
  const guint32 upper_half = (guint32) (magnitude >> 32);
  SCM result;

  result = scm_sum(scm_ash(gh_ulong2scm(upper_half), SCM_MAKINUM(32)),
                   gh_ulong2scm(lower_half));
  
  if(negative_p) {
    return scm_difference(SCM_INUM0, result);
  } else {
    return result;
  }
#endif
}

gint64
gnc_scm_to_gint64(SCM num)
{
#if GUILE_LONG_LONG_OK 
  return scm_num2long_long(num, (char *) SCM_ARG1, "gnc_scm_to_gint64");
#else
  static SCM bits00to15_mask = SCM_BOOL_F;
  SCM magnitude  = scm_abs(num);
  SCM bits;
  unsigned long c_bits;
  long long     c_result = 0;
  int		i;

  /* This doesn't work -- atm (bit-extract 4000 0 32) proves it */
  /*
  SCM lower = scm_bit_extract(magnitude, SCM_MAKINUM(0), SCM_MAKINUM(32));
  */
  
  if (bits00to15_mask == SCM_BOOL_F) {
    bits00to15_mask = gh_ulong2scm(0xFFFF);
    scm_protect_object (bits00to15_mask);
  }

  /*
   * This isn't very complicated (IMHO).  We work from the "top" of
   * the number downwards.  We assume this is no more than a 64-bit
   * number, otherwise it will fail right away.  Anyways, we keep
   * taking the top 16 bits of the number and move it to c_result.
   * Then we 'remove' those bits from the original number and continue
   * with the next 16 bits down, and so on.  -- warlord@mit.edu
   * 2001/02/13
   */
  for (i = 48; i >=0; i-= 16) {
    bits = scm_ash(magnitude, SCM_MAKINUM(-i));
    c_bits = gh_scm2ulong(scm_logand(bits, bits00to15_mask));
    c_result += ((long long)c_bits << i);
    magnitude = scm_difference(magnitude, scm_ash(bits, SCM_MAKINUM(i)));
  }
  
  if(scm_negative_p(num) != SCM_BOOL_F) {
    return(- c_result);
  } 
  else {
    return(c_result);
  }
#endif
}

int
gnc_gh_gint64_p(SCM num)
{
  static int initialized = 0;
  static SCM maxval;
  static SCM minval;

  if(!initialized) {
    /* to be super safe, we have to build these manually because
       though we know that we have gint64's here, we *don't* know how
       to portably specify a 64bit constant to the compiler (i.e. like
       0x7FFFFFFFFFFFFFFF). */
    gint64 tmp;
    
    tmp = 0x7FFFFFFF;
    tmp <<= 32;
    tmp |= 0xFFFFFFFF;
    maxval = gnc_gint64_to_scm(tmp);

    tmp = 0x80000000;
    tmp <<= 32;
    minval = gnc_gint64_to_scm(tmp);

    scm_protect_object(maxval);
    scm_protect_object(minval);
    initialized = 1;
  }
  return((scm_geq_p(num, minval) != SCM_BOOL_F) &&
         (scm_leq_p(num, maxval) != SCM_BOOL_F));
}

gnc_numeric
gnc_scm_to_numeric(SCM gncnum)
{
  static SCM get_num   = SCM_BOOL_F;
  static SCM get_denom = SCM_BOOL_F;
  
  if(get_num == SCM_BOOL_F) {
    get_num = gh_eval_str("gnc:gnc-numeric-num");
  }
  if(get_denom == SCM_BOOL_F) {
    get_denom = gh_eval_str("gnc:gnc-numeric-denom");
  }
  
  return gnc_numeric_create(gnc_scm_to_gint64(gh_call1(get_num, gncnum)),
                            gnc_scm_to_gint64(gh_call1(get_denom, gncnum)));
}

SCM
gnc_numeric_to_scm(gnc_numeric arg)
{
  static SCM maker = SCM_BOOL_F;

  if(maker == SCM_BOOL_F) {
    maker = gh_eval_str("gnc:make-gnc-numeric");
  }
  
  return gh_call2(maker, gnc_gint64_to_scm(gnc_numeric_num(arg)),
                  gnc_gint64_to_scm(gnc_numeric_denom(arg)));
}

int
gnc_numeric_p(SCM arg)
{
  static SCM type_p = SCM_BOOL_F;
  SCM        ret    = SCM_BOOL_F;

  if(type_p == SCM_BOOL_F) {
    type_p = gh_eval_str("gnc:gnc-numeric?");
  }
  ret = gh_call1(type_p, arg);

  if(ret == SCM_BOOL_F) {
    return FALSE;
  }
  else {
    return TRUE;
  }
}


